import { defType, Pointer, WasmStringPtr } from "@thi.ng/wasm-api";
import { __array, __instanceArray, __slice32, __primslice32 } from "@thi.ng/wasm-api/memory";
const __str = (mem, base, isConst = true) => new WasmStringPtr(mem, base, isConst);
const $WebGLContextOpts = defType(1, 9, (mem, base) => {
  return {
    get alpha() {
      return mem.u8[base];
    },
    get antialias() {
      return mem.u8[base + 1];
    },
    get depth() {
      return mem.u8[base + 2];
    },
    get desynchronized() {
      return mem.u8[base + 3];
    },
    get failIfMajorPerformanceCaveat() {
      return mem.u8[base + 4];
    },
    get powerPreference() {
      return mem.u8[base + 5];
    },
    get premultipliedAlpha() {
      return mem.u8[base + 6];
    },
    get preserveDrawingBuffer() {
      return mem.u8[base + 7];
    },
    get stencil() {
      return mem.u8[base + 8];
    }
  };
});
var WebGLPowerPreference = /* @__PURE__ */ ((WebGLPowerPreference2) => {
  WebGLPowerPreference2[WebGLPowerPreference2["default"] = 0] = "default";
  WebGLPowerPreference2[WebGLPowerPreference2["high_performance"] = 1] = "high_performance";
  WebGLPowerPreference2[WebGLPowerPreference2["low_power"] = 2] = "low_power";
  return WebGLPowerPreference2;
})(WebGLPowerPreference || {});
const $ShaderSpec = defType(4, 32, (mem, base) => {
  let $vs, $fs;
  return {
    get vs() {
      return $vs || ($vs = __str(mem, base));
    },
    get fs() {
      return $fs || ($fs = __str(mem, base + 4));
    },
    get attribs() {
      return __slice32(mem, $ShaderAttribSpec, base + 8);
    },
    get varying() {
      return __slice32(mem, $ShaderVaryingSpec, base + 16);
    },
    get uniforms() {
      return __slice32(mem, $ShaderUniformSpec, base + 24);
    }
  };
});
const $ShaderAttribSpec = defType(4, 8, (mem, base) => {
  let $name;
  return {
    get name() {
      return $name || ($name = __str(mem, base));
    },
    get type() {
      return mem.u8[base + 4];
    }
  };
});
const $ShaderVaryingSpec = defType(4, 8, (mem, base) => {
  let $name;
  return {
    get name() {
      return $name || ($name = __str(mem, base));
    },
    get type() {
      return mem.u8[base + 4];
    }
  };
});
const $ShaderUniformSpec = defType(16, 32, (mem, base) => {
  let $name;
  return {
    get name() {
      return $name || ($name = __str(mem, base));
    },
    get type() {
      return mem.u8[base + 4];
    },
    get default() {
      return $UniformValue(mem).instance(base + 16);
    }
  };
});
const $ModelSpec = defType(4, 48, (mem, base) => {
  return {
    get attribs() {
      return __slice32(mem, $ModelAttribSpec, base);
    },
    get instances() {
      return __slice32(mem, $ModelAttribSpec, base + 8);
    },
    get uniforms() {
      return __slice32(mem, $ModelUniform, base + 16);
    },
    get textures() {
      return __primslice32(mem, mem.i32, base + 24, 2);
    },
    get shader() {
      return mem.i32[base + 32 >>> 2];
    },
    get num() {
      return mem.u32[base + 36 >>> 2];
    },
    get numInstances() {
      return mem.u32[base + 40 >>> 2];
    },
    get mode() {
      return mem.u8[base + 44];
    }
  };
});
const $ModelAttribSpec = defType(4, 28, (mem, base) => {
  let $name;
  return {
    get name() {
      return $name || ($name = __str(mem, base));
    },
    get data() {
      return $ModelAttribData(mem).instance(base + 4);
    },
    get type() {
      return mem.u16[base + 12 >>> 1];
    },
    get offset() {
      return mem.u32[base + 16 >>> 2];
    },
    get stride() {
      return mem.u32[base + 20 >>> 2];
    },
    get size() {
      return mem.u32[base + 24 >>> 2];
    }
  };
});
const $ModelAttribData = defType(4, 8, (mem, base) => {
  return {
    get u8() {
      return __primslice32(mem, mem.i8, base, 0);
    },
    get i8() {
      return __primslice32(mem, mem.u8, base, 0);
    },
    get i16() {
      return __primslice32(mem, mem.i16, base, 1);
    },
    get u16() {
      return __primslice32(mem, mem.u16, base, 1);
    },
    get u32() {
      return __primslice32(mem, mem.i32, base, 2);
    },
    get i32() {
      return __primslice32(mem, mem.u32, base, 2);
    },
    get f32() {
      return __primslice32(mem, mem.f32, base, 2);
    }
  };
});
const $AttribUpdateSpec = defType(4, 16, (mem, base) => {
  return {
    get data() {
      return $ModelAttribData(mem).instance(base);
    },
    get type() {
      return mem.u16[base + 8 >>> 1];
    },
    get offset() {
      return mem.u32[base + 12 >>> 2];
    }
  };
});
const $ModelUniform = defType(16, 32, (mem, base) => {
  let $name;
  return {
    get name() {
      return $name || ($name = __str(mem, base));
    },
    get type() {
      return mem.u8[base + 4];
    },
    get value() {
      return $UniformValue(mem).instance(base + 16);
    }
  };
});
const $UniformValue = defType(16, 16, (mem, base) => {
  return {
    get float() {
      return mem.f32[base >>> 2];
    },
    get vec2() {
      const addr = base >>> 2;
      return mem.f32.subarray(addr, addr + 2);
    },
    get vec3() {
      const addr = base >>> 2;
      return mem.f32.subarray(addr, addr + 3);
    },
    get vec4() {
      const addr = base >>> 2;
      return mem.f32.subarray(addr, addr + 4);
    }
  };
});
const $TextureSpec = defType(4, 28, (mem, base) => {
  return {
    get img() {
      return $ImageData(mem).instance(base);
    },
    get width() {
      return mem.u16[base + 8 >>> 1];
    },
    get height() {
      return mem.u16[base + 10 >>> 1];
    },
    get depth() {
      return mem.u16[base + 12 >>> 1];
    },
    get format() {
      return mem.u16[base + 14 >>> 1];
    },
    get target() {
      return mem.u16[base + 16 >>> 1];
    },
    get type() {
      return mem.u16[base + 18 >>> 1];
    },
    get filter() {
      return mem.u16[base + 20 >>> 1];
    },
    get wrap() {
      return mem.u16[base + 22 >>> 1];
    },
    get imgType() {
      return mem.u8[base + 24];
    }
  };
});
const $ImageData = defType(4, 8, (mem, base) => {
  return {
    get none() {
      return mem.u32[base >>> 2];
    },
    get u8() {
      return __primslice32(mem, mem.u8, base, 0);
    },
    get u16() {
      return __primslice32(mem, mem.u16, base, 1);
    },
    get u32() {
      return __primslice32(mem, mem.u32, base, 2);
    },
    get f32() {
      return __primslice32(mem, mem.f32, base, 2);
    }
  };
});
var ImageType = /* @__PURE__ */ ((ImageType2) => {
  ImageType2[ImageType2["none"] = 0] = "none";
  ImageType2[ImageType2["u8"] = 1] = "u8";
  ImageType2[ImageType2["u16"] = 2] = "u16";
  ImageType2[ImageType2["u32"] = 3] = "u32";
  ImageType2[ImageType2["f32"] = 4] = "f32";
  return ImageType2;
})(ImageType || {});
var ShaderAttribType = /* @__PURE__ */ ((ShaderAttribType2) => {
  ShaderAttribType2[ShaderAttribType2["float"] = 0] = "float";
  ShaderAttribType2[ShaderAttribType2["int"] = 1] = "int";
  ShaderAttribType2[ShaderAttribType2["uint"] = 2] = "uint";
  ShaderAttribType2[ShaderAttribType2["vec2"] = 3] = "vec2";
  ShaderAttribType2[ShaderAttribType2["vec3"] = 4] = "vec3";
  ShaderAttribType2[ShaderAttribType2["vec4"] = 5] = "vec4";
  ShaderAttribType2[ShaderAttribType2["mat22"] = 6] = "mat22";
  ShaderAttribType2[ShaderAttribType2["mat33"] = 7] = "mat33";
  ShaderAttribType2[ShaderAttribType2["mat44"] = 8] = "mat44";
  return ShaderAttribType2;
})(ShaderAttribType || {});
var UniformType = /* @__PURE__ */ ((UniformType2) => {
  UniformType2[UniformType2["float"] = 0] = "float";
  UniformType2[UniformType2["int"] = 1] = "int";
  UniformType2[UniformType2["uint"] = 2] = "uint";
  UniformType2[UniformType2["vec2"] = 3] = "vec2";
  UniformType2[UniformType2["vec3"] = 4] = "vec3";
  UniformType2[UniformType2["vec4"] = 5] = "vec4";
  UniformType2[UniformType2["mat22"] = 6] = "mat22";
  UniformType2[UniformType2["mat33"] = 7] = "mat33";
  UniformType2[UniformType2["mat44"] = 8] = "mat44";
  UniformType2[UniformType2["sampler2D"] = 9] = "sampler2D";
  UniformType2[UniformType2["sampler3D"] = 10] = "sampler3D";
  UniformType2[UniformType2["samplerCube"] = 11] = "samplerCube";
  return UniformType2;
})(UniformType || {});
export {
  $AttribUpdateSpec,
  $ImageData,
  $ModelAttribData,
  $ModelAttribSpec,
  $ModelSpec,
  $ModelUniform,
  $ShaderAttribSpec,
  $ShaderSpec,
  $ShaderUniformSpec,
  $ShaderVaryingSpec,
  $TextureSpec,
  $UniformValue,
  $WebGLContextOpts,
  ImageType,
  ShaderAttribType,
  UniformType,
  WebGLPowerPreference
};
